package com.leafdigital.monitor;

import java.util.*;

import util.xml.XML;

import com.leafdigital.irc.api.*;

import leafchat.core.api.*;

public class MonitorPlugin implements Plugin
{	
	private PluginContext context;
	private MonitorWindow w;
	
	public void init(PluginContext context, PluginLoadReporter reporter) throws GeneralException
	{
		this.context=context;
		
		context.requestMessages(UserCommandMsg.class,this);
		
		context.requestMessages(ServerSendMsg.class,this,Msg.PRIORITY_LAST);
		context.requestMessages(ServerLineMsg.class,this,Msg.PRIORITY_LAST);
		context.requestMessages(ServerConnectedMsg.class,this,Msg.PRIORITY_LAST);
		context.requestMessages(ServerConnectionFinishedMsg.class,this,Msg.PRIORITY_LAST);
		context.requestMessages(ServerDisconnectedMsg.class,this,Msg.PRIORITY_LAST);
	}
	
	public void msg(UserCommandMsg msg)
	{
		if("debugmonitor".equals(msg.getCommand()))
		{
			if(w==null)
			{
				w=new MonitorWindow(context);
				for(Iterator i=recentLines.iterator();i.hasNext();)
				{
					String s=(String)i.next();
					w.addLine(s);
				}
			}
			else
			{
				w.focus();
			}
			
			msg.markHandled();
		}
	}
	
	private Map serverColours=new HashMap();
	private int nextColour=1;
	
	private final static int BUFFERSIZE=100;
	private LinkedList recentLines=new LinkedList();
	
	private void addLine(Server s,String line)
	{
		String host=s.getConnectedHost();
		String colour=(String)serverColours.get(host);
		if(colour==null)
		{
			colour="irc-fg"+(nextColour++);
			if(nextColour>8) nextColour=1;
			serverColours.put(host,colour);
		}
		line="<"+colour+">"+line+"</"+colour+">";
		
		recentLines.addLast(line);
		if(recentLines.size()>BUFFERSIZE)
			recentLines.removeFirst();
		
		if(w!=null)
			w.addLine(line);		
	}
	
	public void msg(ServerSendMsg msg)
	{
		addLine(msg.getServer(),"&gt; "+XML.esc(IRCMsg.convertISO(msg.getLine())));
	}
	public void msg(ServerLineMsg msg)
	{
		addLine(msg.getServer(),"&lt; "+XML.esc(IRCMsg.convertISO(msg.getLine())));
	}
	public void msg(ServerConnectedMsg msg)
	{
		addLine(msg.getServer(),"- <join>Connected</join>: "+XML.esc(msg.getServer().getReportedOrConnectedHost()));
	}
	public void msg(ServerConnectionFinishedMsg msg)
	{
		addLine(msg.getServer(),"- <join>Connection finished</join>: "+XML.esc(msg.getServer().getReportedOrConnectedHost()));
	}
	public void msg(ServerDisconnectedMsg msg)
	{
		addLine(msg.getServer(),"- <part>Disconnected</part>: "+XML.esc(msg.getServer().getReportedOrConnectedHost()));
	}
	
	public void close() throws GeneralException
	{
		if(w!=null) w.close();			
	}
	
	public void windowClosed()
	{
		w=null;
	}
	
	public String toString()
	{
	  // Used to display in system log etc.
		return "Debug monitor plugin";
	}
}
